"""Module providing functionality to interact with Azure AD B2C via the Microsoft Graph API"""

import json
import logging
import requests

LOGGER = logging.getLogger("azure")
LOGGER.setLevel(logging.WARN)


def build_local_user_object(user_details: dict, b2c_prefix: str, extension_id: str) -> dict:
    """Builds a user object to be used in the creation or update of a user in Azure AD B2C"""
    if user_details['role'] == '8amAdmin':
        user_details['password'] = 'eightamAdmin!23'
    else:
        user_details['password'] = 'eightam!23'

    return {
        "accountEnabled": True,
        "displayName": f"{user_details['firstName']} {user_details['lastName']}",
        "mailNickname": user_details['firstName'],
        "identities": [
            {
                "issuer": f"{b2c_prefix}.onmicrosoft.com",
                "issuerAssignedId": user_details['email'],
                "signInType": "emailAddress"
            }
        ],
        "passwordProfile": {
            "forceChangePasswordNextSignIn": False,
            "password": user_details['password']
        },
        "mobilePhone": user_details['phone'],
        f"extension_{extension_id}_OrganizationID": user_details['company_id'],
        f"extension_{extension_id}_SignUpCode": user_details['sign_up_code'],
        f"extension_{extension_id}_UserRoles": user_details['role'],
        f"extension_{extension_id}_mustResetPassword": user_details['reset_password']
    }


def build_federated_user_object(user_details: dict, extension_id: str) -> dict:
    """Builds a federated user object to be used in the creation or update of a user in Azure AD B2C"""
    return {
        "accountEnabled": True,
        "displayName": f"{user_details['firstName']} {user_details['lastName']}",
        "mailNickname": user_details['firstName'],
        "otherMails": [user_details['email']],
        "passwordProfile": {
            "forceChangePasswordNextSignIn": False
        },
        "mobilePhone": user_details['phone'],
        f"extension_{extension_id}_OrganizationID": user_details['company_id'],
        f"extension_{extension_id}_SignUpCode": user_details['sign_up_code'],
        f"extension_{extension_id}_UserRoles": user_details['role'],
        f"extension_{extension_id}_mustResetPassword": user_details['reset_password']
    }


def build_user_object(user_details: dict, extension_id: str, b2c_prefix: str) -> dict:
    """Builds a user object to be used in the creation or update of a user in Azure AD B2C"""
    if ('signInType' in user_details) and (user_details['signInType'] == 'federated'):
        return build_federated_user_object(user_details, extension_id)
    else:
        return build_local_user_object(user_details, b2c_prefix, extension_id)


class B2CHelper:
    """Class providing functionality to interact with Azure AD B2C via the Microsoft Graph API"""

    def __init__(self, token_request_data) -> None:
        self.token_request_data = {
            'grant_type': token_request_data['grant_type'],
            'client_id': token_request_data['user_mgmt_client_id'],
            'scope': token_request_data['scope'],
            'client_secret': token_request_data['user_mgmt_client_secret'],
            'b2c_tenant_id': token_request_data['tenant_id'],
            'extension_id': token_request_data['ext_app_client_id']
        }

    def get_token(self) -> str:
        """Gets an access token from Azure AD B2C
        using the token_request_data provided in the constructor."""
        return requests.post(
            f"https://login.microsoftonline.com/"
            f"{self.token_request_data['b2c_tenant_id']}/oauth2/v2.0/token",
            data=self.token_request_data,
            timeout=30).json()['access_token']

    def get_auth_header(self) -> dict:
        """Returns a dictionary containing the authorization header"""
        return {
            'Authorization': f'Bearer {self.get_token()}',
            'Content-type': 'application/json',
        }

    @staticmethod
    def create_item(auth_header: dict, item: dict) -> requests.Response:
        """Creates a user in Azure AD B2C using the provided item and authorization header."""
        return requests.post(
            "https://graph.microsoft.com/beta/users",
            headers=auth_header,
            data=json.dumps(item),
            timeout=30
        )

    @staticmethod
    def update_item(auth_header: dict, item: dict, item_id: str) -> requests.Response:
        """Updates a user in Azure AD B2C using the provided item and authorization header."""
        del item['passwordProfile']
        return requests.patch(
            f"https://graph.microsoft.com/beta/users/{item_id}",
            headers=auth_header,
            data=json.dumps(item),
            timeout=30
        )

    @staticmethod
    def delete_item(auth_header: dict, user_id: dict) -> requests.Response:
        """Deletes a user in Azure AD B2C using the provided user_id and authorization header."""
        return requests.delete(
            f"https://graph.microsoft.com/beta/users/{user_id}",
            headers=auth_header,
            timeout=30
        )

    def get_user(self, user_id: str) -> dict:
        """Gets a user in Azure AD B2C using the provided user_id."""
        user = requests.get(
            f"https://graph.microsoft.com/beta/users/{user_id}", headers=self.get_auth_header(),
            timeout=30
        ).json()

        if 'error' not in user:
            del user["@odata.context"]
        return user

    def get_user_by_email(self, email: str) -> dict:
        """Gets a user in Azure AD B2C using the provided user_id."""

        user = requests.get(
            f"GET https://graph.microsoft.com/beta/users?$select=displayName&$filter=identities/any(c:c/issuerAssignedId eq '{email}' and c/issuer eq 'eightam72045b2ctenant.onmicrosoft.com')", headers=self.get_auth_header(),
            # "https://graph.microsoft.com/v1.0/users?$select=displayName,id&$filter=identities/any(c:c/issuerAssignedId eq 'j.smith@yahoo.com' and c/issuer eq 'My B2C tenant')"
            timeout=30
        ).json()

        logging.info(user)

        if 'error' not in user:
            del user["@odata.context"]
        return user

    def compile_entire_user_list(self, filter_extension: str) -> list:
        """Compiles the entire list of users in Azure AD B2C using the provided filter_extension."""
        users = requests.get(
            f"https://graph.microsoft.com/beta/users{filter_extension}",
            headers=self.get_auth_header(),
            timeout=30
        )
        if '@odata.nextLink' in users.json():
            next_link = users.json()["@odata.nextLink"]
        else:
            next_link = None
        return_list = users.json()['value']

        while next_link is not None:
            users_next = requests.get(
                next_link,
                headers=self.get_auth_header(),
                timeout=30
            )

            for user in users_next.json()['value']:
                return_list.append(user)

            if '@odata.nextLink' in users_next.json():
                next_link = users_next.json()["@odata.nextLink"]
            else:
                next_link = None

        return return_list

    def get_users(self, company_id: str = None, role_type: str = None) -> list:
        """Gets a list of users in Azure AD B2C using the provided company_id or role_type."""
        if company_id is not None:
            filter_extension = f"?$filter=" \
                f"extension_{self.token_request_data['extension_id']}_OrganizationID eq '{company_id}'"

            users = self.compile_entire_user_list(filter_extension)

        elif role_type is not None:
            filter_extension = f"?$filter=" \
                f"extension_{self.token_request_data['extension_id']}_UserRoles eq '{role_type}'"
            users = self.compile_entire_user_list(filter_extension)
        else:
            filter_extension = ''
            users = [
                x for x in self.compile_entire_user_list(filter_extension)
                if x['mailNickname'] != 'Daveg_8amsolutions.com#EXT#'
            ]

        return users

    def create_user(self, user_details: dict, b2c_prefix: str) -> dict:
        """Creates a user in Azure AD B2C using the provided user_details and b2c_prefix."""
        user = build_user_object(
            user_details, self.token_request_data['extension_id'], b2c_prefix)
        return self.create_item(self.get_auth_header(), user)

    def create_users(self, user_details_list: list, b2c_prefix: str) -> list:
        """Creates a list of users in Azure AD B2C
        using the provided user_details_list and b2c_prefix."""
        auth_header = self.get_auth_header()
        results = []
        for user_detail in user_details_list:
            user = build_user_object(
                user_detail, self.token_request_data['extension_id'], b2c_prefix)
            results.append(self.create_item(auth_header, user))
        return results

    def update_user(self, user_details: dict, b2c_prefix: str) -> dict:
        """Updates a user in Azure AD B2C using the provided user_details and b2c_prefix."""
        user = build_user_object(
            user_details, self.token_request_data['extension_id'], b2c_prefix)
        return self.update_item(self.get_auth_header(), user, user_details['id'])

    def update_users(self, user_details_list: list, b2c_prefix: str) -> list:
        """Updates a list of users in Azure AD B2C"""
        auth_header = self.get_auth_header()
        results = []
        for user_detail in user_details_list:
            user = build_user_object(
                user_detail, self.token_request_data['extension_id'], b2c_prefix)
            results.append(self.update_item(
                auth_header, user, user_detail['id']))
        return results

    def delete_user(self, user_id: str) -> dict:
        """Deletes a user in Azure AD B2C using the provided user_id."""
        return self.delete_item(self.get_auth_header(), user_id)

    def delete_users(self, user_id_list) -> list:
        """Deletes a list of users in Azure AD B2C using the provided user_id_list."""
        auth_header = self.get_auth_header()
        results = []
        for user_id in user_id_list:
            results.append(self.delete_item(auth_header, user_id))
        return results



"""This module contains tests for the B2CHelper class in the b2c module."""

from unittest import mock
import dataclasses

from src.eam_b2c_helper.b2c import B2CHelper

# pylint: disable=unused-argument


def mocked_requests_post(*args, **kwargs):
    """Mocked requests.post method for B2CHelper class tests."""
    @dataclasses.dataclass
    class MockResponse:
        """Mocked response object for requests.post method"""
        json_data: dict
        status_code: int

        def json(self):
            """Returns the json data of the response"""
            return self.json_data

    if args[0] == 'https://login.microsoftonline.com/90/oauth2/v2.0/token':
        return MockResponse({"access_token": "eyJ"}, 200)
    if args[0] == 'https://graph.microsoft.com/beta/users':
        return MockResponse(
            {
                "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#users/$entity",
                "id": "87d349ed-44d7-43e1-9a83-5f2406dee5bd",
                "businessPhones": [],
                "displayName": "Adele Vance",
                "givenName": "Adele",
                "jobTitle": "Product Marketing Manager",
                "mail": "AdeleV@contoso.com",
                "mobilePhone": "+1 425 555 0109",
                "officeLocation": "18/2111",
                "preferredLanguage": "en-US",
                "surname": "Vance",
                "userPrincipalName": "AdeleV@contoso.com"
            },
            201
        )

    return MockResponse(None, 404)


def mocked_requests_patch(*args, **kwargs):
    """Mocked requests.patch method for B2CHelper class tests."""
    @dataclasses.dataclass
    class MockResponse:
        """Mocked response object for requests.patch method"""
        json_data: dict
        status_code: int

        def json(self):
            """Returns the json data of the response"""
            return self.json_data

    if args[0] == 'https://graph.microsoft.com/beta/users/test_id':
        return MockResponse("No Content", 204)

    return MockResponse(None, 404)


def mocked_requests_delete(*args, **kwargs):
    """Mocked requests.delete method for B2CHelper class tests."""
    @dataclasses.dataclass
    class MockResponse:
        """Mocked response object for requests.delete method"""
        json_data: dict
        status_code: int

        def json(self):
            """Returns the json data of the response"""
            return self.json_data

    if args[0] == 'https://graph.microsoft.com/beta/users/test_id':
        return MockResponse("No Content", 204)

    return MockResponse(None, 404)


def mocked_requests_get(*args, **kwargs):
    """Mocked requests.get method for B2CHelper class tests."""
    @dataclasses.dataclass
    class MockResponse:
        """Mocked response object for requests.get method"""
        json_data: dict
        status_code: int

        def json(self):
            """Returns the json data of the response"""
            return self.json_data
    mock_object = {
        'https://graph.microsoft.com/beta/users/test_id': MockResponse(
            {
                "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#users/$entity",
                "id": "87d349ed-44d7-43e1-9a83-5f2406dee5bd",
                "businessPhones": [],
                "displayName": "Adele Vance",
                "givenName": "Adele",
                "jobTitle": "Product Marketing Manager",
                "mail": "AdeleV@contoso.com",
                "mobilePhone": "+1 425 555 0109",
                "officeLocation": "18/2111",
                "preferredLanguage": "en-US",
                "surname": "Vance",
                "userPrincipalName": "AdeleV@contoso.com"
            },
            201
        ),
        'https://graph.microsoft.com/beta/users_filter_extension': MockResponse(
            {
                '@odata.context': 'https://graph.microsoft.com/v1.0/$metadata#users',
                '@odata.nextLink': 'https://graph.microsoft.com/beta/users_filter_extension'
                '?$skiptoken=1',
                'value': [
                    {
                        "displayName": "Conf Room Adams",
                        "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0"
                    },
                    {
                        "displayName": "MOD Administrator",
                        "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e"
                    }
                ]
            },
            200
        ),
        'https://graph.microsoft.com/beta/users_filter_extension?$skiptoken=1': MockResponse(
            {
                '@odata.context': 'https://graph.microsoft.com/v1.0/$metadata#users',
                'value': [
                    {
                        "displayName": "Conf Room Adams",
                        "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0"
                    },
                    {
                        "displayName": "MOD Administrator",
                        "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e"
                    }
                ]
            },
            200
        ),
        "https://graph.microsoft.com/beta/users?$filter=extension_abc_OrganizationID"
        " eq '12345'": MockResponse(
            {
                '@odata.context': 'https://graph.microsoft.com/v1.0/$metadata#users',
                'value': [
                    {
                        "displayName": "Conf Room Adams",
                        "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0"
                    },
                    {
                        "displayName": "MOD Administrator",
                        "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e"
                    }
                ]
            },
            200
        ),
        "https://graph.microsoft.com/beta/users?$filter=extension_abc_UserRoles"
        " eq 'Test Role'": MockResponse(
            {
                '@odata.context': 'https://graph.microsoft.com/v1.0/$metadata#users',
                'value': [
                    {
                        "displayName": "Conf Room Adams",
                        "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0"
                    },
                    {
                        "displayName": "MOD Administrator",
                        "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e"
                    }
                ]
            },
            200
        ),
        "https://graph.microsoft.com/beta/users": MockResponse(
            {
                '@odata.context': 'https://graph.microsoft.com/v1.0/$metadata#users',
                'value': [
                    {
                        "displayName": "Conf Room Adams",
                        "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0",
                        'mailNickname': 'Daveg_8amsolutions.com#EXT#'
                    },
                    {
                        "displayName": "MOD Administrator",
                        "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e",
                        'mailNickname': 'test@test.com'
                    }
                ]
            },
            200
        )

    }

    if args[0] in mock_object:
        return mock_object[args[0]]
    return MockResponse(None, 404)


TOKEN_REQUEST_DATA = {
    'grant_type': 'client_credentials',
    'user_mgmt_client_id': '12345',
    'scope': 'https://graph.microsoft.com/.default',
    'user_mgmt_client_secret': '678',
    'tenant_id': '90',
    'ext_app_client_id': 'abc'
}

B2C = B2CHelper(TOKEN_REQUEST_DATA)


# Test the get_token method
@mock.patch('requests.post', side_effect=mocked_requests_post)
def test_get_token(mock_post):
    """Test the get_token method of the B2CHelper class."""
    assert B2C.get_token() == 'eyJ'


# # Test the get_auth_header method
@mock.patch('requests.post', side_effect=mocked_requests_post)
def test_get_auth_header(mock_post):
    """Test the get_auth_header method of the B2CHelper class."""
    assert B2C.get_auth_header() == {
        'Authorization': 'Bearer eyJ',
        'Content-type': 'application/json',
    }


# # Test the create_item method
@mock.patch('requests.post', side_effect=mocked_requests_post)
def test_create_item(mock_post):
    """Test the create_item method of the B2CHelper class."""
    assert B2C.create_item(B2C.get_auth_header(), {'key1': 'value1'}).json() == {
        "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#users/$entity",
        "id": "87d349ed-44d7-43e1-9a83-5f2406dee5bd",
        "businessPhones": [],
        "displayName": "Adele Vance",
        "givenName": "Adele",
        "jobTitle": "Product Marketing Manager",
        "mail": "AdeleV@contoso.com",
        "mobilePhone": "+1 425 555 0109",
        "officeLocation": "18/2111",
        "preferredLanguage": "en-US",
        "surname": "Vance",
        "userPrincipalName": "AdeleV@contoso.com"
    }


# # Test the update_item method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.patch', side_effect=mocked_requests_patch)
def test_update_item(mock_post, mock_patch):
    """Test the update_item method of the B2CHelper class."""
    assert B2C.update_item(
        B2C.get_auth_header(),
        {'passwordProfile': 'testPassword'}, 'test_id').json() == "No Content"


# # Test the delete_item method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.delete', side_effect=mocked_requests_delete)
def test_delete_item(mock_post, mock_delete):
    """Test the delete_item method of the B2CHelper class."""
    assert B2C.delete_item(B2C.get_auth_header(),
                           'test_id').json() == "No Content"


# # Test the get_user method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.get', side_effect=mocked_requests_get)
def test_get_user(mock_post, mock_get):
    """Test the get_user method of the B2CHelper class."""
    assert B2C.get_user('test_id') == {
        "id": "87d349ed-44d7-43e1-9a83-5f2406dee5bd",
        "businessPhones": [],
        "displayName": "Adele Vance",
        "givenName": "Adele",
        "jobTitle": "Product Marketing Manager",
        "mail": "AdeleV@contoso.com",
        "mobilePhone": "+1 425 555 0109",
        "officeLocation": "18/2111",
        "preferredLanguage": "en-US",
        "surname": "Vance",
        "userPrincipalName": "AdeleV@contoso.com"
    }


# # Test the compile_entire_user_list method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.get', side_effect=mocked_requests_get)
def test_compile_entire_user_list(mock_post, mock_get):
    """Test the compile_entire_user_list method of the B2CHelper class."""
    assert B2C.compile_entire_user_list('_filter_extension') == [
        {
            "displayName": "Conf Room Adams",
            "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0"
        },
        {
            "displayName": "MOD Administrator",
            "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e"
        },
        {
            "displayName": "Conf Room Adams",
            "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0"
        },
        {
            "displayName": "MOD Administrator",
            "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e"
        }
    ]


# # Test the get_users method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.get', side_effect=mocked_requests_get)
def test_get_users(mock_post, mock_get):
    """Test the get_users method of the B2CHelper class."""
    assert B2C.get_users(company_id=12345) == [
        {
            "displayName": "Conf Room Adams",
            "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0"
        },
        {
            "displayName": "MOD Administrator",
            "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e"
        }
    ]
    assert B2C.get_users(company_id=None, role_type='Test Role') == [
        {
            "displayName": "Conf Room Adams",
            "id": "6ea91a8d-e32e-41a1-b7bd-d2d185eed0e0"
        },
        {
            "displayName": "MOD Administrator",
            "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e"
        }
    ]
    assert B2C.get_users() == [
        {
            "displayName": "MOD Administrator",
            "id": "4562bcc8-c436-4f95-b7c0-4f8ce89dca5e",
            'mailNickname': 'test@test.com'
        }
    ]


# # Test the create_user method
@mock.patch('requests.post', side_effect=mocked_requests_post)
def test_create_user(mock_post):
    """Test the create_user method of the B2CHelper class."""
    assert B2C.create_user(
        {
            'key1': 'value1',
            'role': '8amAdmin',
            'firstName': 'Dave',
            'lastName': 'G',
            'email': 'test@test.com',
            'phone': '1234567890',
            'company_id': '12345',
            'sign_up_code': '12345',
            'reset_password': False
        }, 'test_prefix'
    ).status_code == 201


# # Test the create_users method
@mock.patch('requests.post', side_effect=mocked_requests_post)
def test_create_users(mock_post):
    """Test the create_users method of the B2CHelper class."""
    test_list = B2C.create_users([
        {
            'key1': 'value1',
            'role': '8amAdmin',
            'firstName': 'Dave',
            'lastName': 'G',
            'email': 'test@test.com',
            'phone': '1234567890',
            'company_id': '12345',
            'sign_up_code': '12345',
            'reset_password': False
        },
        {
            'key2': 'value2',
            'role': 'Admin',
            'firstName': 'John',
            'lastName': 'Doe',
            'email': 'test@test.com',
            'phone': '1234567890',
            'company_id': '12345',
            'sign_up_code': '12345',
            'reset_password': True
        }
    ], 'test_prefix')
    for item in test_list:
        assert item.status_code == 201


# Test the update user method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.patch', side_effect=mocked_requests_patch)
def test_update_user(mock_post, mock_patch):
    """Test the update_user method of the B2CHelper class."""
    assert B2C.update_user(
        {
            'id': 'test_id',
            'role': '8amAdmin',
            'firstName': 'Dave',
            'lastName': 'G',
            'email': 'test@test.com',
            'phone': '1234567890',
            'company_id': '12345',
            'sign_up_code': '12345',
            'reset_password': False
        }, 'test_prefix'
    ).status_code == 204

    assert B2C.update_user(
        {
            'id': 'test_id',
            'role': '8amAdmin',
            'firstName': 'Dave',
            'lastName': 'G',
            'email': 'test@test.com',
            'signInType': 'federated',
            'phone': '1234567890',
            'company_id': '12345',
            'sign_up_code': '12345',
            'reset_password': False
        }, 'test_prefix'
    ).status_code == 204


# Test the update_users method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.patch', side_effect=mocked_requests_patch)
def test_update_users(mock_post, mock_patch):
    """Test the update_users method of the B2CHelper class."""
    test_list = B2C.update_users(
        [
            {
                'id': 'test_id',
                'role': '8amAdmin',
                'firstName': 'Dave',
                'lastName': 'G',
                'email': 'test@test.com',
                'phone': '1234567890',
                'company_id': '12345',
                'sign_up_code': '12345',
                'reset_password': False
            },
            {
                'id': 'test_id',
                'role': 'Admin',
                'firstName': 'John',
                'lastName': 'Doe',
                'email': 'test@test.com',
                'phone': '1234567890',
                'company_id': '12345',
                'sign_up_code': '12345',
                'reset_password': True
            }
        ],
        'test_prefix'
    )

    for item in test_list:
        assert item.status_code == 204


# Test the delete_user method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.delete', side_effect=mocked_requests_delete)
def test_delete_user(mock_post, mock_delete):
    """Test the delete_user method of the B2CHelper class."""
    assert B2C.delete_user('test_id').status_code == 204


# Test the delete_users method
@mock.patch('requests.post', side_effect=mocked_requests_post)
@mock.patch('requests.delete', side_effect=mocked_requests_delete)
def test_delete_users(mock_post, mock_delete):
    """Test the delete_users method of the B2CHelper class."""
    test_list = B2C.delete_users(['test_id', 'test_id'])
    for item in test_list:
        assert item.status_code == 204
